//$Header: /project/A3Pro/conf/ini.c,v 1.7 2003/04/17 08:18:43 vinci Exp $
#include <stdio.h>

#include <stdlib.h>
#include <string.h>
#include <fcntl.h>


#include "../include/typedef.h"
#include "../include/ini.h"

#ifndef PATH_MAX
# define PATH_MAX	1024
#endif

//#define HAVE_OS2_H

#if defined(HAVE_OS2_H)
# define DIR_SEP	";"
# define PATH_SEP	'\\'
#else
# define DIR_SEP	":"
# define PATH_SEP	'/'
#endif

/** @hideinitializer */
#define STRINGIFY1(x)	#x
/** @hideinitializer */
#define STRINGIFY(x)	STRINGIFY1(x)

#define DEFAULT_DIRS	"." DIR_SEP STRINGIFY(PATH_SANE_CONFIG_DIR)




char *
strsep (char **stringp, const char *delim)
{
	char *begin, *end;
	
	begin = *stringp;
	if (! begin || *begin == '\0')
		return NULL;
	
	/* Find the end of the token.  */
	end = strpbrk (begin, delim);
	if (end)
    {
		/* Terminate the token and set *STRINGP past NUL character.  */
		*end++ = '\0';
		*stringp = end;
    }
	else
		/* No more delimiters; this is the last token.  */
		*stringp = NULL;
	
	return begin;
}

FILE *
ini_open (const char *filename)
{
	char  *next, *dir, result[PATH_MAX];
	FILE *fp = 0;
	size_t len;
	void *mem = 0;
	char* dir_list=0;


	if (!dir_list)
    {
		dir_list = getenv ("SANE_CONFIG_DIR");
		if (dir_list)
		{
			len = strlen (dir_list);
			if ((len > 0) && (dir_list[len - 1] == DIR_SEP[0]))
			{
				/* append default search directories: */
				mem = malloc (len + sizeof (DEFAULT_DIRS));
				
				memcpy (mem, dir_list, len);
				memcpy ((char *) mem + len, DEFAULT_DIRS, sizeof (DEFAULT_DIRS));
				dir_list = mem;
			}
		}
		else
		{
			return NULL;
			//dir_list = DEFAULT_DIRS;
		}
    }

	for (next = dir_list; (dir = strsep (&next, DIR_SEP)) != 0; )
    {
		sprintf (result ,"%s%c%s", dir, PATH_SEP, filename);
		fp = fopen (result, "r");
		if (fp)
		{
		  break;
		}
    }
	
	if (mem)
		free(mem);
	
	return fp;
}

/** GET OPTION FROM ini FILE
 **/
 void  Trim( char * SrcStr )
{
  int i,j,k;

  i = j = k = 0;
  for( ; *(SrcStr+i) == ' ' ; i++ ) {;}
  for( j=strlen(SrcStr)-1 ; *(SrcStr+j) == ' '|| *(SrcStr+j)=='\n'|| *(SrcStr+j) == '\0'; j-- ) {;}
  for( k=i ; k<=j ; k++ )
      *(SrcStr+k-i) = *(SrcStr+k);
  
  *(SrcStr+j-i+1) = '\0';

  return;
}

///////////////////////////
//read string
/////////////////////////
int OptionReadString(char * lpchTag, char * lpchTarItem , char* Def_Value, char*buf, int len,char* IniFileName)
{
 
  char * lpchOption = NULL;
  char * lpchSrcItem = NULL;
  char chLine[256];
  char * lpchLine = NULL;
  FILE * lpfp = NULL;
  int FoundTag;
	  
 
  if( ( lpfp = ini_open( IniFileName) ) == NULL ) 
  {
  //    printf("The %s file doesn't exist\n" , IniFileName);
      memcpy(buf,Def_Value,len);
	  return INI_NOFILE;
  }
   FoundTag = FALSE;
  while(!feof(lpfp))
  {
      fgets(chLine, 256, lpfp );
	 
	  
	  if(!feof(lpfp))
      {
		  Trim(chLine);
		   
        if(chLine[0] == '#')continue;
		  
		  if(FoundTag == FALSE) 
          { 
			  if(strcmp(chLine,lpchTag) == 0) 
				  {
					  FoundTag = TRUE;
				  } 
          }
		  else if(FoundTag == TRUE)
		  {
		  lpchLine = strtok(chLine, "=");
			  lpchSrcItem = lpchLine;
			  do {
				  lpchLine = strtok(NULL, "=");
				  if(lpchLine) { lpchOption = lpchLine; }
			  } while(lpchLine);
			  
			  if (lpchSrcItem != NULL && lpchOption != NULL)
			  {
				  if(strcmp(lpchSrcItem, lpchTarItem) ==0)
				  {
			          memcpy(buf,lpchOption,len);
					  fclose(lpfp);
					  return INI_DONE;
					  
				  }
			  }
			  
			  lpchLine = NULL;
			  lpchOption = NULL;
		  }
      }
  }
  memcpy(buf,Def_Value,len);
  fclose( lpfp );
  return INI_DONE;
}


int OptionReadInt(char * lpchTag, char * lpchTarItem ,int Def_Value ,char* IniFileName)
{

	char * lpchOption = NULL;
	char * lpchSrcItem = NULL;
	char chLine[256];
	char * lpchLine = NULL;
	FILE * lpfp = NULL;
	int FoundTag;
	
	
	if( ( lpfp =ini_open(IniFileName) ) == NULL ) 
	{
//		printf("The %s file doesn't exist\n" , IniFileName);
		return Def_Value;
	}
	
	FoundTag = FALSE;
	while(!feof(lpfp))
	{
		fgets(chLine, 256, lpfp );
		if(!feof(lpfp))
		{
			Trim(chLine);

			if(chLine[0] == '#')continue;

			if(FoundTag == FALSE) 
			{ 
				if(strcmp(lpchTag, chLine)==0) { FoundTag = TRUE; } 
			}
			else if(FoundTag == TRUE)
			{
				lpchLine = strtok(chLine, "=");
				lpchSrcItem = lpchLine;
				do {
					lpchLine = strtok(NULL, "=");
					if(lpchLine) { lpchOption = lpchLine; }
				} while(lpchLine);
				
				if (lpchSrcItem != NULL && lpchOption != NULL)
				{
					if(strcmp(lpchSrcItem, lpchTarItem)==0)
					{
					
					    int temp = atoi(lpchOption);
						
                        fclose(lpfp);
						return temp;
						
					}
				}
				
				lpchLine = NULL;
				lpchOption = NULL;
			}
		}
	}
	fclose( lpfp );
	return Def_Value;
}

