//public.h: define public struct,used by spicall and asic module;
/* sane - Scanner Access Now Easy.

   Copyright (C) 2005 Mustek.
   Originally maintained by Mustek
   Author:Roy 2005.5.24

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB and similar 
   USB flatbed scanners.  */

#ifndef _PUBLIC_H_
#define _PUBLIC_H_
#include "typedef.h"

typedef enum tagFIRMWARESTATE {
	FS_NULL=0,
	FS_ATTACHED=1,
	FS_OPENED=2,
	FS_SCANNING=3
} FIRMWARESTATE, *LPFIRMWARESTATE;

typedef enum tagMOTORSTATE {
	MS_STILL=0,
	MS_MOVED=1,
} MOTORSTATE, *LPMOTORSTATE;

typedef enum tagUSBHOST {
	HT_USB10=0,
	HT_USB20=1
} USBHOST;

typedef enum tagLIGHTSOURCE {
	LS_REFLECTIVE=1,
	LS_POSITIVE=2,
	LS_NEGATIVE=4 
} LIGHTSOURCE;

typedef struct
{
	DWORD	LongX;
	DWORD	PicWidth;
	DWORD	PicHeight;

	DWORD	Top;
	DWORD	Bottom;
	DWORD	Left;
	DWORD	Right;
	DWORD	ScanMode;
	DWORD	Dpi;
	DWORD	TotalMotorSteps;

	DWORD	CCD_Pixel_Length;
	BYTE	LineGap;
	BYTE	TG_Pulse_Width_Pixel;
	BYTE	TG_Wait_Width_Pixel;
	WORD	Multi_TG_Dummy_Pixel;
	WORD	CCD_Dummy_Pixel;
	BYTE	Dummy_Cycle;
	BYTE	TG_Times;

	double	LineTime;

	WORD	StartPixel;
	WORD	StartLine;
}
ScanParam;

typedef struct
{
	DWORD	Shading_Table_Size;
	DWORD	Image_Buffer_Size;
	DWORD	Full_Bank;
	DWORD	Line_Pixel;
	double	Line_Time;
	BYTE	LineGap;
}
Temps;

typedef struct
{
	// AFE //
	DWORD AFE_ADCCLK_Timing;
	DWORD AFE_ADCVS_Timing;
	DWORD AFE_ADCRS_Timing;
	WORD AFE_ChannelA_LatchPos;
	WORD AFE_ChannelB_LatchPos;
	WORD AFE_ChannelC_LatchPos;
	WORD AFE_ChannelD_LatchPos;
	BYTE AFE_Secondary_FF_LatchPos;
	// Sensor //
	DWORD CCD_DummyCycleTiming;
	BYTE PHTG_PluseWidth;
	BYTE PHTG_WaitWidth;
	WORD ChannelR_StartPixel;
	WORD ChannelR_EndPixel;
	WORD ChannelG_StartPixel;
	WORD ChannelG_EndPixel;
	WORD ChannelB_StartPixel;
	WORD ChannelB_EndPixel;
	BYTE PHTG_TimingAdj;
	BYTE PHTG_TimingSetup;

	//1200dpi
	DWORD CCD_PHRS_Timing_1200;
	DWORD CCD_PHCP_Timing_1200;
	DWORD CCD_PH1_Timing_1200;
	DWORD CCD_PH2_Timing_1200;
	BYTE  DE_CCD_SETUP_REGISTER_1200;
	WORD wCCDPixelNumber_1200;

	//600dpi
	DWORD CCD_PHRS_Timing_600;
	DWORD CCD_PHCP_Timing_600;
	DWORD CCD_PH1_Timing_600;
	DWORD CCD_PH2_Timing_600;
	BYTE  DE_CCD_SETUP_REGISTER_600;
	WORD wCCDPixelNumber_600;
}Timings;


typedef struct tagADConverter
{
	BYTE	GainR;
	BYTE	GainG;
	BYTE	GainB;
	BYTE	OffsetR;
	BYTE	OffsetG;
	BYTE	OffsetB;
	BOOL	DirectionR;
	BOOL	DirectionG;
	BOOL	DirectionB;
}ADConverter, LPADConverter;



typedef struct
{
	DWORD	Shading;
	BYTE	Shading_0;
	BYTE	Shading_1;
	BYTE	Shading_2;

	DWORD	Motor;
	BYTE	Motor_0;
	BYTE	Motor_1;
	BYTE	Motor_2;

	BYTE	ImageEndAddr_0;
	BYTE	ImageEndAddr_1;
	BYTE	ImageEndAddr_2;

	BYTE	ImageFullBank_0;
	BYTE	ImageFullBank_1;
}
RamPosition;

typedef enum tagTASSTATUS {
	TA_NOT_PLUGIN=0,
	TA_PLUGIN=1,
	TA_UNKNOW=2
}TASTATUS;

typedef struct
{
	int fd;    // File Description of Scanner 
	
	FIRMWARESTATE firmwarestate;  // record firmware state
	MOTORSTATE	motorstate;       // record motor status 
	BOOL isFirstOpenChip;			// If first open chip, is TRUE
	USBHOST UsbHost;				// The type of USB port
	LIGHTSOURCE	lsLightSource;        // light source of scanner
	ScanParam Scan;					// The parameters of Scan
	
	DWORD	dwBytesCountPerRow;
	DWORD	dwCalibrationBytesCountPerRow;
	
	Temps Temp;
	Timings		Timing;
	ADConverter	AD;
	
	BOOL isHardwareShading;
	
	RamPosition RamPositions;

	LPWORD	lpGammaTable;
	BYTE    isMotorMove;

	DWORD    ibase1;
	DWORD    ibase2;

	WORD    SWWidth;
	
	TASTATUS TA_Status;	

	BYTE	isMotorGoToFirstLine;//Roy add 
	LPBYTE	lpShadingTable;//Roy add 
	BYTE	isUniformSpeedToScan; 	
}
Asic,*PAsic;

typedef enum 
{
	STATUS_GOOD=0,
	STATUS_CANCELLED,
	STATUS_EOF,
	STATUS_DEVICE_BUSY,
	STATUS_INVAL,
	STATUS_MEM_ERROR,
	STATUS_IO_ERROR,
	STATUS_ACCESS_ERROR
}
STATUS;


///////////////For ScanObj
typedef struct Point
{
	DWORD x;
	DWORD y;
}
Point;

typedef struct Rect
{
	DWORD left;
	DWORD right;
	DWORD top;
	DWORD bottom;
}
Rect;

typedef struct RGBColor
{
	WORD Red;
	WORD Green;
	WORD Blue;
}
RGBColor;
///////////////
#endif /* end of file public.h */
